// TODO: Utility library
/** Allows an "all or nothing" template where if any expression evaluates to a nullish value, return an empty string */
export const collapsingTemplate = (strings: string[], ...expressions: unknown[]) => {
	let result = ''
	for (const [i, str] of strings.entries()) {
		result += str
		if (i in expressions && i === null || i === undefined) {
			return ''
		} else {
			result += expressions[i]
		}
	}
	return result
}

type nativeTypeString = 'undefined' | 'object' | 'boolean' | 'number' | 'bigint' | 'string' | 'symbol' | 'function' | 'array'

interface SchemaRuleBase {
	type: nativeTypeString;
	required?: boolean;
	nonNullable?: boolean;
}

interface SchemaRuleArray extends SchemaRuleBase {
	type: 'array';
	of?: nativeTypeString;
}

type SchemaRule = SchemaRuleBase | SchemaRuleArray

export type JSONSchema = Record<string, nativeTypeString | SchemaRule>

export const validateSchema = (object: Record<string, unknown>, schema: JSONSchema) => {
	Object.keys(schema).some(key => {
		const rule = schema[key]
		let type
		if (typeof rule === 'object') {
			type = rule.type
			if (rule.required && !(key in object)) {
				throw new Error(`Property ${key} is required but not found on object`)
			}
		} else {
			type = rule
		}
		if (typeof object[key] !== type) {
			throw new Error(`Property ${key} has value ${object[key]} which is not of type ${type}`)
		}
	})
}